/*
	* 	Net Image Filter -- a netpbm based image filtering service
	* 					by Marco Scheurer, Sen:te
	*
	* 	Version 1.0
	* 
	* 	Copyright 1995, Sente Sarl. All rights reserved.
	*
	*		This notice may not be removed from this source code.
	*
	*	This object is included in the MiscKit by permission from the author
	*	and its use is governed by the MiscKit license, found in the file
	*	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
	*	for a list of all applicable permissions and restrictions.
	*
*/

#import "NetImageConverter.h"
#import <appkit/appkit.h>

#define DEFAULTS_DB	"NetImageFilter"

static NXDefaultsVector FilterDefaults = {
	{"netpbm", "/usr/local/netpbm"},
	{"Debug", "NO"},
	{NULL}
};


/*
	*	Information about a filter executable (such as netpbm binaries)
	*
	*	The User Data associated with a file extension and a pbm executable is [x/]y
	*	(see services file)
	*	"x" is used as the key in the NXDefaultsVector to retrieve 
	*	the path of executable named y.
	*	If x does not exist it is assumed that y is in the users's paths
	*	for executables.
	*
	*	Would be nicer with FoundationKit. 
*/

NXAtom filterFullPathFromData (const char *data)
{
	char path[MAXPATHLEN + 1] = "";
	char *separator = index (data, '/');

	if (separator && *separator)
	{
		char key[MAXPATHLEN + 1];
		strncpy (key, data, separator - data);
		key [separator - data] = 0;
		sprintf (path, "%s/%s", NXGetDefaultValue (DEFAULTS_DB, key), separator + 1);
	}
	else
	{
		sprintf (path, "%s", data);
	}
	return NXUniqueString(path);
}


@implementation NetImageConverter
+ initialize
{
    NXRegisterDefaults (DEFAULTS_DB, FilterDefaults);
	return self;
}


- (BOOL) isDebugOn
{
	const char *defaultValue = NXGetDefaultValue (DEFAULTS_DB, "Debug");
	return (defaultValue && !strcmp(defaultValue, "YES"));
}


- anyToTIFF:pasteboard userData:(const char *)userData error:(char **)errorMessage
{
/*
 *	PRECONDITIONS
 *	We make several assumptions here, based on the fact that this method is supposed to
 *	be called by the system with a meaningful pasteboard:
 *		- We assume that there is exactly one type available:
 *				NXTypeFilenamePboardType:<a file extension>
 *		- We assume that there is exactly one file available, not a tab separated list.
 *
 *	If the filtering is succesfull pasteboard contains data for the NXTIFFPboardType
 */

	NXAtom	foundType = [pasteboard types][0];	

	if (foundType)
	{
		char	*destPath = NULL;
		char	*data = NULL;
		int		length = 0;
		
		[pasteboard readType:foundType data:&data length:&length];

		if (data && length && userData && *userData)
		{
			char commandFormat[] = "%s %s | %s > %s";
			char command [4 * (MAXPATHLEN + 1) + strlen (commandFormat)];
			char sourcePath [MAXPATHLEN + 1];
			char destTemplate[] = "/tmp/NetImageFilter-@@@@@@.tiff";
			
	    	strncpy (sourcePath, data, length);
			sourcePath[length] = 0;
			
			destPath = NXGetTempFilename (destTemplate, index(destTemplate, '@') - destTemplate);

			sprintf (
				command, 
				commandFormat, 
				filterFullPathFromData (userData),
				sourcePath,
				filterFullPathFromData ("netpbm/pnmtotiff"),
				destPath
			);

			if ([self isDebugOn])
			{
				freopen ("/tmp/console.log", "a", stderr);
				fprintf (stderr, "%s\n", command);
			}
			else
			{
				freopen ("/dev/null", "w", stderr);
			}
			system (command);
		}
		[pasteboard deallocatePasteboardData:data length:length];
		
		if (destPath)
		{
			NXStream *tiffStream = NXMapFile (destPath, NX_READONLY);
			if (tiffStream)
			{
				[pasteboard declareTypes:&NXTIFFPboardType num:1 owner:self];
				[pasteboard writeType:NXTIFFPboardType fromStream:tiffStream];
				NXCloseMemory (tiffStream, NX_FREEBUFFER);
				tiffStream = NULL;
			}
			unlink (destPath);
		}
	}
	return self;
}
@end
